//  Kratos Multi-Physics - ThermalDEM Application
//
//  License:       BSD License
//                 Kratos default license: kratos/license.txt
//
//  Main authors:  Rafael Rangel (rrangel@cimne.upc.edu)
//

#if !defined(KRATOS_THERMAL_DEM_APPLICATION_VARIABLES_H_INCLUDED)
#define KRATOS_THERMAL_DEM_APPLICATION_VARIABLES_H_INCLUDED

// System includes

// External includes
#include "includes/kratos_application.h"
#include "includes/define.h"
#include "includes/variables.h"

// Project includes
#include "custom_strategies/schemes/thermal_dem_integration_scheme.h"
#include "custom_utilities/numerical_integration_method.h"
#include "custom_constitutive/heat_exchange_mechanism.h"
#include "custom_constitutive/heat_generation_mechanism.h"
#include "custom_constitutive/real_contact/real_contact_model.h"

namespace Kratos
{
  // Variables
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, ThermalDEMIntegrationScheme::Pointer, THERMAL_INTEGRATION_SCHEME_POINTER)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, NumericalIntegrationMethod::Pointer,  NUMERICAL_INTEGRATION_METHOD_POINTER)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, HeatExchangeMechanism::Pointer,       DIRECT_CONDUCTION_MODEL_POINTER)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, HeatExchangeMechanism::Pointer,       INDIRECT_CONDUCTION_MODEL_POINTER)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, HeatExchangeMechanism::Pointer,       CONVECTION_MODEL_POINTER)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, HeatExchangeMechanism::Pointer,       RADIATION_MODEL_POINTER)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, HeatGenerationMechanism::Pointer,     GENERATION_MODEL_POINTER)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, RealContactModel::Pointer,            REAL_CONTACT_MODEL_POINTER)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, std::string,                          THERMAL_INTEGRATION_SCHEME_NAME)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, std::string,                          NUMERICAL_INTEGRATION_METHOD_NAME)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, std::string,                          DIRECT_CONDUCTION_MODEL_NAME)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, std::string,                          INDIRECT_CONDUCTION_MODEL_NAME)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, std::string,                          CONVECTION_MODEL_NAME)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, std::string,                          RADIATION_MODEL_NAME)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, std::string,                          REAL_CONTACT_MODEL_NAME)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, std::string,                          VORONOI_METHOD_NAME)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, std::string,                          POROSITY_METHOD_NAME)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, bool,                                 AUTO_SOLVE_FREQUENCY_OPTION)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, bool,                                 MOTION_OPTION)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, bool,                                 DIRECT_CONDUCTION_OPTION)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, bool,                                 INDIRECT_CONDUCTION_OPTION)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, bool,                                 CONVECTION_OPTION)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, bool,                                 RADIATION_OPTION)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, bool,                                 HEAT_GENERATION_OPTION)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, bool,                                 GENERATION_SLIDING_OPTION)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, bool,                                 GENERATION_ROLLING_OPTION)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, bool,                                 GENERATION_DAMPING_OPTION)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, bool,                                 REAL_CONTACT_OPTION)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, bool,                                 FIXED_TEMPERATURE)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, bool,                                 ADIABATIC)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, int,                                  THERMAL_FREQUENCY)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               HEATFLUX)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               THERMAL_CONDUCTIVITY)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               REAL_YOUNG_MODULUS_RATIO)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               HEATSOURCE)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               MIN_CONDUCTION_DISTANCE)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               MAX_CONDUCTION_DISTANCE)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               CONDUCTION_RADIUS)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               ISOTHERMAL_CORE_RADIUS)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               MAX_RADIATION_DISTANCE)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               HEAT_GENERATION_RATIO)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               AVERAGE_POROSITY)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               ALPHA_SHAPE_PARAMETER)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               INTEGRAL_TOLERANCE)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               FLUID_LAYER_THICKNESS)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               FLUID_DENSITY)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               FLUID_VISCOSITY)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               FLUID_THERMAL_CONDUCTIVITY)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               FLUID_HEAT_CAPACITY)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, double,                               FLUID_TEMPERATURE)
  KRATOS_DEFINE_APPLICATION_VARIABLE(THERMAL_DEM_APPLICATION, Vector,                               FLUID_VELOCITY)

  // Flags
  class DEMThermalFlags
  {
    public:
      KRATOS_DEFINE_LOCAL_APPLICATION_FLAG(THERMAL_DEM_APPLICATION, IS_ADIABATIC);
      KRATOS_DEFINE_LOCAL_APPLICATION_FLAG(THERMAL_DEM_APPLICATION, IS_SINTERING);
  };
} // namespace Kratos

#endif // KRATOS_THERMAL_DEM_APPLICATION_VARIABLES_H_INCLUDED defined
